@@ -141,7 +141,7 @@ def run(prg):
         print >>sys.stderr, e
         return 2
 
-    except oscerr.RuntimeError, e:
+    except oscerr.ExtRuntimeError, e:
         print >>sys.stderr, e.msg
         return 1
 
@@ -149,12 +149,24 @@ def run(prg):
         print >>sys.stderr, e
         return 1
 
-    except (oscerr.PackageExists, oscerr.PackageMissing), e:
+    except (oscerr.PackageExists, oscerr.PackageMissing, oscerr.WorkingCopyInconsistent), e:
         print >>sys.stderr, e.msg
         return 1
 
-    except AttributeError, e:
-        print >>sys.stderr, e
+    except oscerr.PackageInternalError, e:
+        import traceback
+        print >>sys.stderr, 'a package internal error occured\n' \
+            'please file a bug and attach your current package working copy ' \
+            'and the following traceback to it:'
+        print >>sys.stderr, e.msg
+        traceback.print_exc(file=sys.stderr)
+
+    except oscerr.PackageError, e:
+        print >>sys.stderr, e.msg
+        return 1
+
+    except PackageError, e:
+        print >>sys.stderr, '%s:' % e.fname, e.msg
         return 1
 
     except RPMError, e:
@@ -177,8 +189,4 @@ def run(prg):
         print >>sys.stderr, e
         return 1
 
-    except PackageError, e:
-        print >>sys.stderr, e.msg
-        return 1
-
 # vim: sw=4 et
