@@ -46,11 +46,12 @@ can_also_build = {
              's390x':  ['s390' ],
              'ppc64':  [                        'ppc', 'ppc64' ],
              'sh4':    [                                                                                               'sh4' ],
-             'i386':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
-             'i586':   [                'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
-             'i686':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
-             'x86_64': ['i686', 'i586', 'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mips64' ],
+             'i386':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
+             'i586':   [                'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
+             'i686':   [        'i586',         'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
+             'x86_64': ['i686', 'i586', 'i386', 'ppc', 'ppc64',  'armv4l', 'armv5el', 'armv6el', 'armv7el', 'armv8el', 'sh4', 'mips', 'mipsel' ],
              'sparc64': ['sparc64v', 'sparcv9v', 'sparcv9', 'sparcv8', 'sparc'],
+             'parisc': ['hppa'],
              }
 
 # real arch of this machine
@@ -58,6 +59,9 @@ hostarch = os.uname()[4]
 if hostarch == 'i686': # FIXME
     hostarch = 'i586'
 
+if hostarch == 'parisc':
+    hostarch = 'hppa'
+
 class Buildinfo:
     """represent the contents of a buildinfo file"""
 
@@ -118,6 +122,8 @@ class Buildinfo:
         self.vminstall_list = [ dep.name for dep in self.deps if dep.vminstall ]
         self.cbinstall_list = [ dep.name for dep in self.deps if dep.cbinstall ]
         self.cbpreinstall_list = [ dep.name for dep in self.deps if dep.cbpreinstall ]
+        self.cbpreignore_list = [ dep.name for dep in self.deps if dep.cbpreignore ]
+        self.cbignore_list = [ deb.name for dep in self.deps if dep.cbignore ]
         self.preinstall_list = [ dep.name for dep in self.deps if dep.preinstall ]
         self.runscripts_list = [ dep.name for dep in self.deps if dep.runscripts ]
 
@@ -129,11 +135,12 @@ class Buildinfo:
         return False
 
     def remove_dep(self, name):
+        # we need to iterate over all deps because if this a
+        # kiwi build the same package might appear multiple times
         for i in self.deps:
-            if i.name == name:
+            # only remove those which are needed for the build itself
+            if i.name == name and not i.noinstall:
                 self.deps.remove(i)
-                return True
-        return False
 
 
 class Pac:
@@ -148,7 +155,7 @@ class Pac:
                   'version', 'release',
                   'project', 'repository',
                   'preinstall', 'vminstall', 'noinstall', 'runscripts',
-                  'cbinstall', 'cbpreinstall',
+                  'cbinstall', 'cbpreinstall', 'cbpreignore', 'cbignore',
                  ]:
             self.mp[i] = node.get(i)
 
@@ -158,7 +165,7 @@ class Pac:
         self.mp['arch'] = node.get('arch') or self.mp['buildarch']
 
         # this is not the ideal place to check if the package is a localdep or not
-        localdep = self.mp['name'] in localpkgs
+        localdep = self.mp['name'] in localpkgs and not self.mp['noinstall']
         if not localdep and not (node.get('project') and node.get('repository')):
             raise oscerr.APIError('incomplete information for package %s, may be caused by a broken project configuration.'
                                   % self.mp['name'] )
@@ -555,7 +562,7 @@ def main(apiurl, opts, argv):
             print 'Getting buildconfig from server and store to %s' % bc_filename
             if not bc_file:
                 bc_file = open(bc_filename, 'w')
-            bc_file.write(get_buildconfig(apiurl, prj, pac, repo, arch))
+            bc_file.write(get_buildconfig(apiurl, prj, repo))
             bc_file.flush()
     except urllib2.HTTPError, e:
         if e.code == 404:
@@ -631,7 +638,7 @@ def main(apiurl, opts, argv):
         # OBS 1.5 and before has no downloadurl defined in buildinfo
         if bi.downloadurl:
             urllist.append(bi.downloadurl + '/%(extproject)s/%(extrepository)s/%(arch)s/%(filename)s')
-    if not opts.cpio_bulk_download:
+    if opts.disable_cpio_bulk_download:
         urllist.append( '%(apiurl)s/build/%(project)s/%(repository)s/%(repoarch)s/%(repopackage)s/%(repofilename)s' )
 
     fetcher = Fetcher(cachedir = config['packagecachedir'],
@@ -639,7 +646,7 @@ def main(apiurl, opts, argv):
                       api_host_options = config['api_host_options'],
                       offline = opts.noinit or opts.offline,
                       http_debug = config['http_debug'],
-                      enable_cpio = opts.cpio_bulk_download,
+                      enable_cpio = not opts.disable_cpio_bulk_download,
                       cookiejar=cookiejar)
 
     # implicitly trust the project we are building for
@@ -749,7 +756,7 @@ def main(apiurl, opts, argv):
             print 'Skipping verification of package signatures'
         else:
             print 'Verifying integrity of cached packages'
-            verify_pacs([ i.fullfilename for i in bi.deps ], bi.keys)
+            verify_pacs(bi)
 
     elif bi.pacsuffix == 'deb':
         if vm_type == "xen" or vm_type == "kvm" or vm_type == "lxc":
@@ -770,6 +777,8 @@ def main(apiurl, opts, argv):
     rpmlist.append('vminstall: ' + ' '.join(bi.vminstall_list) + '\n')
     rpmlist.append('cbinstall: ' + ' '.join(bi.cbinstall_list) + '\n')
     rpmlist.append('cbpreinstall: ' + ' '.join(bi.cbpreinstall_list) + '\n')
+    rpmlist.append('cbpreignore: ' + ' '.join(bi.cbpreinstall_list) + '\n')
+    rpmlist.append('cbignore: ' + ' '.join(bi.cbpreinstall_list) + '\n')
     rpmlist.append('runscripts: ' + ' '.join(bi.runscripts_list) + '\n')
 
     rpmlist_file = NamedTemporaryFile(prefix='rpmlist.')
