@@ -79,16 +79,20 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'build-vmdisk-swapsize': '', # optional for VM builds
 
              'build-jobs': os.sysconf('SC_NPROCESSORS_ONLN'), # compile with N jobs
+             'builtin_signature_check': '1', # by default use builtin check for verify pkgs
              'icecream': '0',
 
              'debug': '0',
              'http_debug': '0',
+             'http_full_debug': '0',
              'verbose': '1',
              'traceback': '0',
              'post_mortem': '0',
              'use_keyring': '1',
              'gnome_keyring': '1',
              'cookiejar': '~/.osc_cookiejar',
+             # fallback for osc build option --no-verify
+             'no_verify': '0',
              # enable project tracking by default
              'do_package_tracking': '1',
              # default for osc build
@@ -101,7 +105,7 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              'checkout_no_colon': '0',
              # local files to ignore with status, addremove, ....
              # local files to ignore with status, addremove, ....
-             'exclude_glob': '.osc CVS .svn .* _linkerror *~ #*# *.orig *.bak *.changes.*',
+             'exclude_glob': '.osc CVS .svn .* _linkerror *~ #*# *.orig *.bak *.changes.vctmp.*',
              # keep passwords in plaintext. If you see this comment, your osc
              # already uses the encrypted password, and only keeps them in plain text
              # for backwards compatibility. Default will change to 0 in future releases.
@@ -109,7 +113,7 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              # limit the age of requests shown with 'osc req list'.
              # this is a default only, can be overridden by 'osc req list -D NNN'
              # Use 0 for unlimted.
-             'request_list_days': 30,
+             'request_list_days': 0,
              # check for unversioned/removed files before commit
              'check_filelist': '1',
              # External scripts to validate sources, esp before commit. This is a directory
@@ -119,6 +123,8 @@ DEFAULTS = { 'apiurl': 'https://api.opensuse.org',
              # what to do with the source package if the submitrequest has been accepted
              'submitrequest_on_accept_action': '',
              'request_show_interactive': '0',
+             'submitrequest_accepted_template': '',
+             'submitrequest_declined_template': '',
              'linkcontrol': '0',
 
              # Maintenance defaults to OBS instance defaults
@@ -133,7 +139,7 @@ config = DEFAULTS.copy()
 
 boolean_opts = ['debug', 'do_package_tracking', 'http_debug', 'post_mortem', 'traceback', 'check_filelist', 'plaintext_passwd',
     'checkout_no_colon', 'check_for_request_on_action', 'linkcontrol', 'show_download_progress', 'request_show_interactive',
-    'use_keyring', 'gnome_keyring']
+    'use_keyring', 'gnome_keyring', 'no_verify', 'builtin_signature_check', 'http_full_debug']
 
 api_host_options = ['user', 'pass', 'passx', 'aliases', 'http_headers', 'email', 'sslcertck', 'cafile', 'capath', 'trusted_prj']
 
@@ -231,6 +237,9 @@ apiurl = %(apiurl)s
 # show HTTP traffic useful for debugging
 #http_debug = 1
 
+# Skip signature verification of packages used for build.
+#no_verify = 1
+
 # jump into the debugger in case of errors
 #post_mortem = 1
 
@@ -250,6 +259,16 @@ apiurl = %(apiurl)s
 # nothing is specified the API default is used
 #submitrequest_on_accept_action = cleanup|update|noupdate
 
+# template for an accepted submitrequest
+#submitrequest_accepted_template = Hi %%(who)s,\\n
+# thanks for working on:\\t%%(dst_project)s/%%(dst_package)s.
+# SR %%(reqid)s has been accepted.\\n\\nYour maintainers
+
+# template for a declined submitrequest
+#submitrequest_declined_template = Hi %%(who)s,\\n
+# sorry your SR %%(reqid)s (request type: %%(type)s) for
+# %%(dst_project)s/%%(dst_package)s has been declined because...
+
 #review requests interactively (default: off)
 #request_show_review = 1
 
@@ -308,6 +327,11 @@ def parse_apisrv_url(scheme, apisrv):
 def urljoin(scheme, apisrv):
     return '://'.join([scheme, apisrv])
 
+def is_known_apiurl(url):
+    """returns true if url is a known apiurl"""
+    apiurl = urljoin(*parse_apisrv_url(None, url))
+    return config['api_host_options'].has_key(apiurl)
+
 def get_apiurl_api_host_options(apiurl):
     """
     Returns all apihost specific options for the given apiurl, None if
@@ -318,11 +342,10 @@ def get_apiurl_api_host_options(apiurl):
     # had been mingled into before.  But this works fine for now.
 
     apiurl = urljoin(*parse_apisrv_url(None, apiurl))
-    try:
+    if is_known_apiurl(apiurl):
         return config['api_host_options'][apiurl]
-    except KeyError:
-        raise oscerr.ConfigMissingApiurl('missing credentials for apiurl: \'%s\'' % apiurl,
-                                         '', apiurl)
+    raise oscerr.ConfigMissingApiurl('missing credentials for apiurl: \'%s\'' % apiurl,
+                                     '', apiurl)
 
 def get_apiurl_usr(apiurl):
     """
@@ -342,64 +365,65 @@ def get_apiurl_usr(apiurl):
             % (apiurl, config['user'])
         return config['user']
 
-
-
-def init_basicauth(config):
-    """initialize urllib2 with the credentials for Basic Authentication"""
-
+# workaround m2crypto issue:
+# if multiple SSL.Context objects are created
+# m2crypto only uses the last object which was created.
+# So we need to build a new opener everytime we switch the
+# apiurl (because different apiurls may have different
+# cafile/capath locations)
+def _build_opener(url):
     from osc.core import __version__
-    import cookielib
+    import urllib2
+    import sys
+    global config
+    apiurl = urljoin(*parse_apisrv_url(None, url))
+    if not _build_opener.__dict__.has_key('last_opener'):
+        _build_opener.last_opener = (None, None)
+    if apiurl == _build_opener.last_opener[0]:
+        return _build_opener.last_opener[1]
 
-    if config['api_host_options'][config['apiurl']]['sslcertck']:
+    # workaround for http://bugs.python.org/issue9639
+    authhandler_class = urllib2.HTTPBasicAuthHandler
+    if sys.version_info >= (2, 6, 6) and sys.version_info < (2, 7, 1) \
+        and not 'reset_retry_count' in dir(urllib2.HTTPBasicAuthHandler):
+        print >>sys.stderr, 'warning: your urllib2 version seems to be broken. ' \
+            'Using a workaround for http://bugs.python.org/issue9639'
+        class OscHTTPBasicAuthHandler(urllib2.HTTPBasicAuthHandler):
+            def http_error_401(self, *args):
+                response = urllib2.HTTPBasicAuthHandler.http_error_401(self, *args)
+                self.retried = 0
+                return response
+
+        authhandler_class = OscHTTPBasicAuthHandler
+    elif sys.version_info >= (2, 6, 5) and sys.version_info < (2, 6, 6):
+        # workaround for broken urllib2 in python 2.6.5: wrong credentials
+        # lead to an infinite recursion
+        class OscHTTPBasicAuthHandler(urllib2.HTTPBasicAuthHandler):
+            def retry_http_basic_auth(self, host, req, realm):
+                # don't retry if auth failed
+                if req.get_header(self.auth_header, None) is not None:
+                    return None
+                return urllib2.HTTPBasicAuthHandler.retry_http_basic_auth(self, host, req, realm)
+
+        authhandler_class = OscHTTPBasicAuthHandler
+
+    options = config['api_host_options'][apiurl]
+    # with None as first argument, it will always use this username/password
+    # combination for urls for which arg2 (apisrv) is a super-url
+    authhandler = authhandler_class( \
+        urllib2.HTTPPasswordMgrWithDefaultRealm())
+    authhandler.add_password(None, apiurl, options['user'], options['pass'])
+
+    if options['sslcertck']:
         try:
             import oscssl
             from M2Crypto import m2urllib2
         except ImportError, e:
             print e
-            raise NoSecureSSLError("M2Crypto is needed to access %s in a secure way.\nPlease install python-m2crypto." % config['apiurl'])
+            raise NoSecureSSLError('M2Crypto is needed to access %s in a secure way.\nPlease install python-m2crypto.' % apiurl)
 
-    import urllib2
-
-
-    global cookiejar
-
-    import sys
-    if sys.version_info < (2, 6):
-        # HTTPS proxy is not supported in old urllib2. It only leads to an error
-        # or, at best, a warning.
-        if 'https_proxy' in os.environ:
-            del os.environ['https_proxy']
-        if 'HTTPS_PROXY' in os.environ:
-            del os.environ['HTTPS_PROXY']
-
-    if config['http_debug']:
-        # brute force
-        def urllib2_debug_init(self, debuglevel=0):
-            self._debuglevel = 1
-        urllib2.AbstractHTTPHandler.__init__ = urllib2_debug_init
-
-    authhandler = urllib2.HTTPBasicAuthHandler( \
-        urllib2.HTTPPasswordMgrWithDefaultRealm())
-
-    cookie_file = os.path.expanduser(config['cookiejar'])
-    cookiejar = cookielib.LWPCookieJar(cookie_file)
-    try:
-        cookiejar.load(ignore_discard=True)
-    except IOError:
-        try:
-            open(cookie_file, 'w').close()
-            os.chmod(cookie_file, 0600)
-        except:
-            #print 'Unable to create cookiejar file: \'%s\'. Using RAM-based cookies.' % cookie_file
-            cookiejar = cookielib.CookieJar()
-
-
-    if config['api_host_options'][config['apiurl']]['sslcertck']:
-        cafile = capath = None
-        if 'capath' in config['api_host_options'][config['apiurl']]:
-            capath = config['api_host_options'][config['apiurl']]['capath']
-        if 'cafile' in config['api_host_options'][config['apiurl']]:
-            cafile = config['api_host_options'][config['apiurl']]['cafile']
+        cafile = options.get('cafile', None)
+        capath = options.get('capath', None)
         if not cafile and not capath:
             for i in ['/etc/pki/tls/cert.pem', '/etc/ssl/certs' ]:
                 if os.path.isfile(i):
@@ -415,15 +439,68 @@ def init_basicauth(config):
         import sys
         print >>sys.stderr, "WARNING: SSL certificate checks disabled. Connection is insecure!\n"
         opener = urllib2.build_opener(urllib2.HTTPCookieProcessor(cookiejar), authhandler)
-
-    urllib2.install_opener(opener)
-
     opener.addheaders = [('User-agent', 'osc/%s' % __version__)]
+    _build_opener.last_opener = (apiurl, opener)
+    return opener
 
-    # with None as first argument, it will always use this username/password
-    # combination for urls for which arg2 (apisrv) is a super-url
-    for host, auth in config['api_host_options'].iteritems():
-        authhandler.add_password(None, host, auth['user'], auth['pass'])
+def init_basicauth(config):
+    """initialize urllib2 with the credentials for Basic Authentication"""
+
+    import cookielib
+    import urllib2
+    import sys
+    import httplib
+    def filterhdrs(meth, ishdr, *hdrs):
+        import re
+        import sys
+        import StringIO
+        # this is so ugly but httplib doesn't use
+        # a logger object or such
+        def new_method(*args, **kwargs):
+            stdout = sys.stdout
+            sys.stdout = StringIO.StringIO()
+            meth(*args, **kwargs)
+            hdr = sys.stdout.getvalue()
+            sys.stdout = stdout
+            for i in hdrs:
+                if ishdr:
+                    hdr = re.sub(r'%s:[^\\r]*\\r\\n' % i, '', hdr)
+                else:
+                    hdr = re.sub(i, '', hdr)
+            sys.stdout.write(hdr)
+        new_method.__name__ = meth.__name__
+        return new_method
+
+    if config['http_debug'] and not config['http_full_debug']:
+        httplib.HTTPConnection.send = filterhdrs(httplib.HTTPConnection.send, True, 'Cookie', 'Authorization')
+        httplib.HTTPResponse.begin = filterhdrs(httplib.HTTPResponse.begin, False, 'header: Set-Cookie.*\n')
+
+    if sys.version_info < (2, 6):
+        # HTTPS proxy is not supported in old urllib2. It only leads to an error
+        # or, at best, a warning.
+        if 'https_proxy' in os.environ:
+            del os.environ['https_proxy']
+        if 'HTTPS_PROXY' in os.environ:
+            del os.environ['HTTPS_PROXY']
+
+    if config['http_debug']:
+        # brute force
+        def urllib2_debug_init(self, debuglevel=0):
+            self._debuglevel = 1
+        urllib2.AbstractHTTPHandler.__init__ = urllib2_debug_init
+
+    cookie_file = os.path.expanduser(config['cookiejar'])
+    global cookiejar
+    cookiejar = cookielib.LWPCookieJar(cookie_file)
+    try:
+        cookiejar.load(ignore_discard=True)
+    except IOError:
+        try:
+            open(cookie_file, 'w').close()
+            os.chmod(cookie_file, 0600)
+        except:
+            #print 'Unable to create cookiejar file: \'%s\'. Using RAM-based cookies.' % cookie_file
+            cookiejar = cookielib.CookieJar()
 
 
 def get_configParser(conffile=None, force_read=False):
@@ -599,6 +676,7 @@ def get_config(override_conffile = None,
                override_apiurl = None,
                override_debug = None,
                override_http_debug = None,
+               override_http_full_debug = None,
                override_traceback = None,
                override_post_mortem = None,
                override_no_keyring = None,
@@ -700,10 +778,13 @@ def get_config(override_conffile = None,
                 except:
                     print "%s: no credentials known" % url
                     password = 'your_password'
-            else:
+            elif password != passwordx.decode('base64').decode('bz2'):
                 if not passwordx:
-                    print "%s: rewriting from plain pass to encoded pass\n" % url
-                    add_section(conffile, url, user, password)
+                    # passx not present
+                    print '%s: rewriting from plain pass to encoded pass\n' % url
+                else:
+                    print '%s: pass and passx mismatch (rewriting from plain pass to encoded pass)\n' % url
+                add_section(conffile, url, user, password)
 
         if cp.has_option(url, 'http_headers'):
             http_headers = cp.get(url, 'http_headers')
@@ -735,6 +816,9 @@ def get_config(override_conffile = None,
         if not 'sslcertck' in api_host_options[apiurl]:
             api_host_options[apiurl]['sslcertck'] = True
 
+        if scheme == 'http':
+            api_host_options[apiurl]['sslcertck'] = False
+
         if cp.has_option(url, 'trusted_prj'):
             api_host_options[apiurl]['trusted_prj'] = cp.get(url, 'trusted_prj').split(' ')
         else:
@@ -768,6 +852,9 @@ def get_config(override_conffile = None,
         config['debug'] = override_debug
     if override_http_debug:
         config['http_debug'] = override_http_debug
+    if override_http_full_debug:
+        config['http_debug'] = override_http_full_debug or config['http_debug']
+        config['http_full_debug'] = override_http_full_debug
     if override_traceback:
         config['traceback'] = override_traceback
     if override_post_mortem:
