@@ -54,7 +54,7 @@ class OscFileGrabber:
 
 class Fetcher:
     def __init__(self, cachedir = '/tmp', api_host_options = {}, urllist = [], http_debug = False,
-                 cookiejar = None, offline = False, enable_cpio = False):
+                 cookiejar = None, offline = False, enable_cpio = True):
         # set up progress bar callback
         if sys.stdout.isatty() and TextMeter:
             self.progress_obj = TextMeter(fo=sys.stdout)
@@ -84,6 +84,51 @@ class Fetcher:
               % (self.curpac, self.curpac.project, errobj.url.split('/')[2])
         return {}
 
+    def __add_cpio(self, pac):
+        prpap = '%s/%s/%s/%s' % (pac.project, pac.repository, pac.repoarch, pac.repopackage)
+        self.cpio.setdefault(prpap, {})[pac.repofilename] = pac
+
+    def __fetch_cpio(self, apiurl):
+        from urllib import quote_plus
+        for prpap, pkgs in self.cpio.iteritems():
+            project, repo, arch, package = prpap.split('/', 3)
+            query = ['binary=%s' % quote_plus(i) for i in pkgs.keys()]
+            query.append('view=cpio')
+            tmparchive = tmpfile = None
+            try:
+                (fd, tmparchive) = tempfile.mkstemp(prefix='osc_build_cpio')
+                (fd, tmpfile) = tempfile.mkstemp(prefix='osc_build')
+                url = makeurl(apiurl, ['build', project, repo, arch, package], query=query)
+                sys.stdout.write("preparing download ...\r")
+                sys.stdout.flush()
+                self.gr.urlgrab(url, filename = tmparchive, text = 'fetching cpio for \'%s\'' % project)
+                archive = cpio.CpioRead(tmparchive)
+                archive.read()
+                for hdr in archive:
+                    # XXX: we won't have an .errors file because we're using
+                    # getbinarylist instead of the public/... route (which is
+                    # routed to getbinaries (but that won't work for kiwi products))
+                    if hdr.filename == '.errors':
+                        archive.copyin_file(hdr.filename)
+                        raise oscerr.APIError('CPIO archive is incomplete (see .errors file)')
+                    if package == '_repository':
+                        pac = pkgs[hdr.filename.rsplit('.', 1)[0]]
+                    else:
+                        # this is a kiwi product
+                        pac = pkgs[hdr.filename]
+                    archive.copyin_file(hdr.filename, os.path.dirname(tmpfile), os.path.basename(tmpfile))
+                    self.move_package(tmpfile, pac.localdir, pac)
+                    # check if we got all packages... (because we've no .errors file)
+                for pac in pkgs.itervalues():
+                    if not os.path.isfile(pac.fullfilename):
+                        raise oscerr.APIError('failed to fetch file \'%s\': ' \
+                            'does not exist in CPIO archive' % pac.repofilename)
+            finally:
+                if not tmparchive is None and os.path.exists(tmparchive):
+                    os.unlink(tmparchive)
+                if not tmpfile is None and os.path.exists(tmpfile):
+                    os.unlink(tmpfile)
+
     def fetch(self, pac, prefix=''):
         # for use by the failure callback
         self.curpac = pac
@@ -92,10 +137,9 @@ class Fetcher:
         mg = MirrorGroup(self.gr, pac.urllist, failure_callback=(self.failureReport,(),{}))
 
         if self.http_debug:
-            print
-            print 'URLs to try for package \'%s\':' % pac
-            print '\n'.join(pac.urllist)
-            print
+            print >>sys.stderr, '\nURLs to try for package \'%s\':' % pac
+            print >>sys.stderr, '\n'.join(pac.urllist)
+            print >>sys.stderr
 
         (fd, tmpfile) = tempfile.mkstemp(prefix='osc_build')
         try:
@@ -106,7 +150,7 @@ class Fetcher:
                 self.move_package(tmpfile, pac.localdir, pac)
             except URLGrabError, e:
                 if self.enable_cpio and e.errno == 256:
-                    self.cpio.setdefault(pac.project, {})[pac.name] = pac
+                    self.__add_cpio(pac)
                     return
                 print
                 print >>sys.stderr, 'Error:', e.strerror
@@ -121,14 +165,6 @@ class Fetcher:
     def move_package(self, tmpfile, destdir, pac_obj = None):
         import shutil
         pkgq = packagequery.PackageQuery.query(tmpfile, extra_rpmtags=(1044, 1051, 1052))
-        arch = pkgq.arch()
-        # SOURCERPM = 1044
-        if pkgq.filename_suffix == 'rpm' and not pkgq.gettag(1044):
-            # NOSOURCE = 1051, NOPATCH = 1052
-            if pkgq.gettag(1051) or pkgq.gettag(1052):
-                arch = "nosrc"
-            else:
-                arch = "src"
         canonname = pkgq.canonname()
         fullfilename = os.path.join(destdir, canonname)
         if pac_obj is not None:
@@ -147,7 +183,6 @@ class Fetcher:
                 sys.exit(1)
 
     def run(self, buildinfo):
-        from urllib import quote_plus
         cached = 0
         all = len(buildinfo.deps)
         for i in buildinfo.deps:
@@ -180,33 +215,8 @@ class Fetcher:
                     print 'Exiting.'
                     sys.exit(0)
                 done += 1
-        for project, pkgs in self.cpio.iteritems():
-            repo = pkgs.values()[0].repository
-            query = [ 'binary=%s' % quote_plus(i) for i in pkgs.keys() ]
-            query.append('view=cpio')
-            try:
-                (fd, tmparchive) = tempfile.mkstemp(prefix='osc_build_cpio')
-                (fd, tmpfile) = tempfile.mkstemp(prefix='osc_build')
-                url = makeurl(buildinfo.apiurl,
-                              ['public/build', project, repo, buildinfo.buildarch, '_repository'],
-                              query=query)
-                sys.stdout.write("preparing download ...\r")
-                sys.stdout.flush()
-                self.gr.urlgrab(url, filename = tmparchive, text = 'fetching cpio for \'%s\'' % project)
-                archive = cpio.CpioRead(tmparchive)
-                archive.read()
-                for hdr in archive:
-                    if hdr.filename == '.errors':
-                        archive.copyin_file(hdr.filename)
-                        raise oscerr.APIError('CPIO archive is incomplete (see .errors file)')
-                    pac = pkgs[hdr.filename.rsplit('.', 1)[0]]
-                    archive.copyin_file(hdr.filename, os.path.dirname(tmpfile), os.path.basename(tmpfile))
-                    self.move_package(tmpfile, pac.localdir, pac)
-            finally:
-                if os.path.exists(tmparchive):
-                    os.unlink(tmparchive)
-                if os.path.exists(tmpfile):
-                    os.unlink(tmpfile)
+
+        self.__fetch_cpio(buildinfo.apiurl)
 
         prjs = buildinfo.projects.keys()
         for i in prjs:
@@ -215,35 +225,30 @@ class Fetcher:
                 os.makedirs(dest, mode=0755)
             dest += '/_pubkey'
 
-            if os.path.exists(dest):
-                buildinfo.keys.append(dest)
-                buildinfo.prjkeys.append(i)
-            else:
-                url = "%s/source/%s/_pubkey" % (buildinfo.apiurl, i)
-                try:
-                    self.gr.urlgrab(url, dest, text="fetching key for %s" % i)
+            url = "%s/source/%s/_pubkey" % (buildinfo.apiurl, i)
+            try:
+                OscFileGrabber().urlgrab(url, dest)
+                if not i in buildinfo.prjkeys: # not that many keys usually
                     buildinfo.keys.append(dest)
                     buildinfo.prjkeys.append(i)
-                except KeyboardInterrupt:
-                    print 'Cancelled by user (ctrl-c)'
-                    print 'Exiting.'
-                    if os.path.exists(dest):
-                        os.unlink(dest)
-                    sys.exit(0)
-                except URLGrabError, e:
-                    if self.http_debug:
-                        print "can't fetch key for %s: %s" %(i, e.strerror)
-                        print "url: %s" % url
-                    else:
-                        print "%s doesn't have a gpg key" % i
+            except KeyboardInterrupt:
+                print 'Cancelled by user (ctrl-c)'
+                print 'Exiting.'
+                if os.path.exists(dest):
+                    os.unlink(dest)
+                sys.exit(0)
+            except URLGrabError, e:
+                if self.http_debug:
+                    print >>sys.stderr, "can't fetch key for %s: %s" %(i, e.strerror)
+                    print >>sys.stderr, "url: %s" % url
 
-                    if os.path.exists(dest):
-                        os.unlink(dest)
+                if os.path.exists(dest):
+                    os.unlink(dest)
 
-                    l = i.rsplit(':', 1)
-                    # try key from parent project
-                    if len(l) > 1 and l[1] and not l[0] in buildinfo.projects:
-                        prjs.append(l[0])
+                l = i.rsplit(':', 1)
+                # try key from parent project
+                if len(l) > 1 and l[1] and not l[0] in buildinfo.projects:
+                    prjs.append(l[0])
 
 def verify_pacs_old(pac_list):
     """Take a list of rpm filenames and run rpm -K on them.
@@ -312,31 +317,29 @@ def verify_pacs_old(pac_list):
             sys.exit(1)
 
 
-def verify_pacs(pac_list, key_list):
+def verify_pacs(bi):
     """Take a list of rpm filenames and verify their signatures.
 
        In case of failure, exit.
        """
 
-    # XXX: remove if new code stable
-    if not conf.config.get('builtin_signature_check', False):
+    pac_list = [ i.fullfilename for i in bi.deps ]
+    if not conf.config['builtin_signature_check']:
         return verify_pacs_old(pac_list)
 
     if not pac_list:
         return
 
-    if not key_list:
-        print "no keys"
-        sys.exit(1)
-        return
+    if not bi.keys:
+        raise oscerr.APIError("can't verify packages due to lack of GPG keys")
 
-    print key_list
+    print "using keys from", ', '.join(bi.prjkeys)
 
     import checker
     failed = False
     checker = checker.Checker()
     try:
-        checker.readkeys(key_list)
+        checker.readkeys(bi.keys)
         for pkg in pac_list:
             try:
                 checker.check(pkg)
