@@ -10,6 +10,7 @@ import M2Crypto.m2urllib2
 import urlparse
 import socket
 import urllib
+import httplib
 
 class TrustedCertStore:
     _tmptrusted = {}
@@ -181,8 +182,11 @@ class myHTTPSHandler(M2Crypto.m2urllib2.HTTPSHandler):
 
         if (target_host != host):
             h = myProxyHTTPSConnection(host = host, appname = self.appname, ssl_context = self.ctx)
+            # M2Crypto.ProxyHTTPSConnection.putrequest expects a fullurl
+            selector = full_url
         else:
             h = myHTTPSConnection(host = host, appname = self.appname, ssl_context = self.ctx)
+            selector = req.get_selector()
         # End our change
         h.set_debuglevel(self._debuglevel)
 
@@ -196,7 +200,7 @@ class myHTTPSHandler(M2Crypto.m2urllib2.HTTPSHandler):
         # request.
         headers["Connection"] = "close"
         try:
-            h.request(req.get_method(), req.get_full_url(), req.data, headers)
+            h.request(req.get_method(), selector, req.data, headers)
             r = h.getresponse()
         except socket.error, err: # XXX what error?
             err.filename = full_url
@@ -236,7 +240,7 @@ class myHTTPSConnection(M2Crypto.httpslib.HTTPSConnection):
     def getPort(self):
         return self.port
 
-class myProxyHTTPSConnection(M2Crypto.httpslib.ProxyHTTPSConnection):
+class myProxyHTTPSConnection(M2Crypto.httpslib.ProxyHTTPSConnection, httplib.HTTPSConnection):
     def __init__(self, *args, **kwargs):
         self.appname = kwargs.pop('appname', 'generic')
         M2Crypto.httpslib.ProxyHTTPSConnection.__init__(self, *args, **kwargs)
@@ -245,6 +249,11 @@ class myProxyHTTPSConnection(M2Crypto.httpslib.ProxyHTTPSConnection):
         M2Crypto.httpslib.ProxyHTTPSConnection._start_ssl(self)
         verify_certificate(self)
 
+    def endheaders(self, *args, **kwargs):
+        if self._proxy_auth is None:
+            self._proxy_auth = self._encode_auth()
+        httplib.HTTPSConnection.endheaders(self, *args, **kwargs)        
+
     # broken in m2crypto: port needs to be an int
     def putrequest(self, method, url, skip_host=0, skip_accept_encoding=0):
         #putrequest is called before connect, so can interpret url and get
